// -*- C++ -*-
// CLASSDOC OFF
// $Id: TemplateFunctions.h,v 1.5 2002/03/30 09:53:06 evc Exp $
// ---------------------------------------------------------------------------
// CLASSDOC ON
//
// This file is a part of the CLHEP - a Class Library for High Energy Physics.
//
// This file contains template definitions of some usefull functions.
//
#ifndef HEP_TEMPLATEFUNCTIONS_H
#define HEP_TEMPLATEFUNCTIONS_H

#include "CLHEP/config/CLHEP.h"

#ifndef CLHEP_MAX_MIN_DEFINED
#define CLHEP_MAX_MIN_DEFINED
#ifdef max
#undef max
#endif
#ifdef min
#undef min
#endif

template <class T>
inline const T& max(const T& a, const T& b) {
  // Break this into two lines to avoid an incorrect warning with
  // Cfront-based compilers.
  const T& retval = a < b ? b : a;
  return retval;
}

template <class T>
inline const T& min(const T& a, const T& b) {
  // Break this into two lines to avoid an incorrect warning with
  // Cfront-based compilers.
  const T& retval = b < a ? b : a;
  return retval;
}
#endif

#ifndef CLHEP_SQR_DEFINED
#define CLHEP_SQR_DEFINED
#ifdef sqr
#undef sqr
#endif
template <class T>
inline T sqr(const T& x) {
  return x*x;
}
#endif

#ifndef CLHEP_ABS_DEFINED
#define CLHEP_ABS_DEFINED
#ifdef abs
#undef abs
#endif
template <class T>
inline T abs(const T& a) {
  return a < 0 ? -a : a;
}
#endif

#endif /* HEP_TEMPLATEFUNCTIONS_H */
